﻿import { CustomButton } from "@docsvision/webclient/Platform/CustomButton";
import { Layout } from "@docsvision/webclient/System/Layout";
import { DocumentLogic } from "./DocumentLogic";
import { States } from "../../Constants/States";
import { Employee } from "@docsvision/webclient/BackOffice/Employee";
import { DisplayStaffUnit } from "@docsvision/webclient/Platform/DisplayStaffUnit";
import { Department } from "@docsvision/webclient/BackOffice/Department";
import { $DepartmentController, $LayoutPartnerController } from "@docsvision/webclient/Generated/DocsVision.WebClient.Controllers";
import { GenModels } from "@docsvision/webclient/Generated/DocsVision.WebClient.Models";
import { MessageBox } from "@docsvision/webclient/Helpers/MessageBox/MessageBox";
import { $UnitsController } from "../../Services/UnitsController";
import { $CompanyController } from "../../Services/CompanyController ";
import { $DocumentsController, DocumentsController } from "../../Services/DocumentsController";
import { Routes } from "../../Constants/Routes";


/**
 * Класс общей логики для эксплуатационной документации
 */
export class ContractM2Logic extends DocumentLogic {


    /**
     * Открытие карточки
     * @param layout
     */
    public async CardOpened(layout: Layout) {
        await super.CardOpened(layout);
    }

    public async setExecutorInfo(layout: Layout) {
        let executorControl = layout.controls.tryGet<Employee>("executor");

        if (!executorControl) { return; }

        let organizationControl = layout.controls.tryGet<Department>("organization");
        let subdivisionControl = layout.controls.tryGet<Department>("subdivision");

        if (!organizationControl || !subdivisionControl) { return; }

        if (!executorControl.value) {
            organizationControl.value = null;
            subdivisionControl.value = null;
        }
        else {
            let departmentSrvc = layout.getService($DepartmentController);
            let documentSrvc = layout.getService($DocumentsController);

            let department = await departmentSrvc.getStaffDepartment(executorControl.value.unitId);

            subdivisionControl.value = department;

            let organization = await documentSrvc.GetRootUnit(department.id);

            organizationControl.value = organization;
        }
    }

}

